"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _lodash = _interopRequireDefault(require("lodash.flow"));

var _getSsrCacheTags = _interopRequireDefault(require("./ssrCache/getSsrCacheTags"));

var _fields = require("@webiny/commodo/fields");

var _repropose = require("@webiny/commodo/repropose");

var _fieldsDate = require("@webiny/commodo/fields-date");

var _fieldsObject = require("@webiny/commodo/fields-object");

var _name = require("@webiny/commodo/name");

var _hooks = require("@webiny/commodo/hooks");

var _functions = require("@webiny/http-handler-ssr/functions");

var _default = ({
  createBase,
  options = {}
} = {}) => (0, _lodash.default)((0, _name.withName)("SsrCache"), (0, _fields.withFields)({
  path: (0, _fields.string)({
    validation(value) {
      if (!value) {
        throw new Error(`SsrCache entry's "path" field is required.`);
      }

      if (!value.startsWith("/")) {
        throw new Error(`SsrCache entry's "path" field must begin with forward slash ("/")`);
      }
    }

  }),
  content: (0, _fields.string)({
    value: null
  }),
  version: (0, _fields.string)({
    value: null,

    validate(value) {
      if (value && value.length > 100) {
        throw Error(`Field "version" cannot have more than 100 characters.`);
      }
    }

  }),
  refreshedOn: (0, _fields.skipOnPopulate)()((0, _fieldsDate.date)()),
  expiresOn: (0, _fields.skipOnPopulate)()((0, _fieldsDate.date)()),
  cacheTags: (0, _fields.skipOnPopulate)()((0, _fieldsObject.object)({
    list: true
  })),
  lastRefresh: (0, _fields.skipOnPopulate)()((0, _fields.fields)({
    value: {},
    instanceOf: (0, _lodash.default)((0, _repropose.withProps)({
      get duration() {
        return this.endedOn - this.startedOn;
      }

    }), (0, _fields.withFields)({
      startedOn: (0, _fieldsDate.date)(),
      endedOn: (0, _fieldsDate.date)(),
      duration: (0, _fields.number)()
    }))()
  }))
}), (0, _hooks.withHooks)({
  beforeSave() {
    this.cacheTags = (0, _getSsrCacheTags.default)(this.content);
  }

}), (0, _repropose.withStaticProps)({
  findByPath(path) {
    return this.findOne({
      query: {
        path
      }
    });
  }

}), (0, _repropose.withProps)({
  get hasExpired() {
    return !this.expiresIn;
  },

  get hasContent() {
    return typeof this.content === "string" && this.content.length > 0;
  },

  get isEmpty() {
    return !this.hasContent;
  },

  get expiresIn() {
    if (!this.expiresOn) {
      return 0;
    } // @ts-ignore


    const expiresIn = this.expiresOn - new Date();
    return expiresIn > 0 ? expiresIn : 0;
  },

  get isRefreshing() {
    return this.lastRefresh.startedOn && !this.lastRefresh.endedOn;
  },

  isStale(currentVersion) {
    if (this.hasExpired) {
      return true;
    }

    return currentVersion && currentVersion !== this.version;
  },

  incrementExpiresOn(seconds = options.cache.staleTtl) {
    this.expiresOn = new Date();
    this.expiresOn.setSeconds(this.expiresOn.getSeconds() + seconds);
    return this;
  },

  async refresh(currentVersion = null) {
    this.version = currentVersion;
    this.lastRefresh.startedOn = new Date();
    this.lastRefresh.endedOn = null;
    this.lastRefresh.duration = null;
    await this.save();
    this.content = await (0, _functions.getSsrHtml)(options.ssrFunction, {
      path: this.path
    });
    this.refreshedOn = new Date();
    this.lastRefresh.endedOn = this.refreshedOn;
    this.lastRefresh.duration = this.lastRefresh.endedOn - this.lastRefresh.startedOn;
    this.incrementExpiresOn(options.cache.ttl);
    await this.save();
  },

  async invalidate() {
    await this.hook("beforeInvalidate");
    this.expiresOn = null;
    this.content = null;
    this.version = null;
    await this.save();
    await this.hook("afterInvalidate");
  }

}))(createBase());

exports.default = _default;
//# sourceMappingURL=ssrCache.model.js.map